@@ -10,7 +10,7 @@
 #include "util.h"
 #include "wiki.h"
 
-#define ADD_ITEM_COUNT 5
+#define ADD_ITEM_COUNT 10
 
 typedef enum _NAVI {
     NAVI_OTHER,
@@ -67,9 +67,9 @@ void output_header(Project* project, char* title, char* script_name, NaviType na
             "\t<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n"
             "\t<meta http-equiv=\"Content-Script-Type\" content=\"text/javascript\" />\n"
             "\t<meta http-equiv=\"Content-Style-type\" content=\"text/css\" />");
-    o(        "\t<title>管理ツール - "); h(project->name); o(" - "); h(title); o("</title>\n");
+    o(        "\t<title>管理ツール - "); h(string_rawstr(project->name)); o(" - "); h(title); o("</title>\n");
     o(      "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/../css/style.css\" />\n", cgiScriptName);
-    o(      "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/%s/setting_file/user.css\" />\n", cgiScriptName, g_project_name_4_url);
+    o(      "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/../index.%s/%s/setting_file/user.css\" />\n", cgiScriptName, get_ext(cgiScriptName), g_project_name_4_url);
     if (script_name) {
         o(  "\t<script type=\"text/javascript\" src=\"%s/../js/prototype.js\"></script>\n", cgiScriptName);
         o(  "\t<script type=\"text/javascript\" src=\"%s/../js/%s\"></script>\n", cgiScriptName, script_name);
@@ -90,7 +90,7 @@ void output_header(Project* project, char* title, char* script_name, NaviType na
     o(      "\t\t<li><a %s href=\"%s/%s/items\">項目設定</a></li>\n", navi == NAVI_ITEM ? "class=\"current\"" : "", cgiScriptName, g_project_name_4_url);
     o(      "\t\t<li><a %s href=\"%s/%s/style\">スタイル設定</a></li>\n", navi == NAVI_STYLE ? "class=\"current\"" : "", cgiScriptName, g_project_name_4_url);
     o(      "\t\t<li><a %s href=\"%s/%s/admin_help\">ヘルプ</a></li>\n", navi == NAVI_ADMIN_HELP ? "class=\"current\"" : "", cgiScriptName, g_project_name_4_url);
-    o(      "\t<li><a href=\"%s/../index.%s/%s/\">", cgiScriptName, get_ext(cgiScriptName), g_project_name_4_url);h(project->name); o("へ</a></li>\n");
+    o(      "\t<li><a href=\"%s/../index.%s/%s/\">", cgiScriptName, get_ext(cgiScriptName), g_project_name_4_url);h(string_rawstr(project->name)); o("へ</a></li>\n");
     o(      "</ul>\n"
             "<br clear=\"all\" />\n");
 }
@@ -138,7 +138,7 @@ void top_action()
     if (strlen(message) > 0) {
         o("<div class=\"complete_message\">"); h(message); o("&nbsp;</div>\n");
     }
-    o("<h2>"); h(project_a->name); o(" 管理ツール</h2>");
+    o("<h2>"); h(string_rawstr(project_a->name)); o(" 管理ツール</h2>");
     project_free(project_a);
     o("<div id=\"main_body\">\n");
     o("<h3>管理ツールメニュー</h3>\n");
@@ -174,19 +174,23 @@ void project_action()
     project_a = db_get_project(db_a, project_a);
     output_header(project_a, "サブプロジェクト設定", "management.js", NAVI_PROJECT);
 
-    o("<h2>%s 管理ツール</h2>", project_a->name);
+    o("<h2>%s 管理ツール</h2>", string_rawstr(project_a->name));
     o("<div id=\"setting_form\">\n");
     o("\t<form id=\"management_form\" action=\"%s/%s/project_submit\" method=\"post\" enctype=\"multipart/form-data\">\n", cgiScriptName, g_project_name_4_url);
     o("\t\t<h3>サブプロジェクト設定</h3>\n");
     o("\t\t<table summary=\"project table\">\n");
     o("\t\t\t<tr>\n");
     o("\t\t\t\t<th>サブプロジェクト名</th>\n");
-    o("\t\t\t\t<td><input type=\"text\" name=\"project.name\" value=\"");h(project_a->name);o("\" maxlength=\"1000\" /></td>\n");
+    o("\t\t\t\t<td><input type=\"text\" name=\"project.name\" value=\"");h(string_rawstr(project_a->name));o("\" maxlength=\"1000\" /></td>\n");
     o("\t\t\t</tr>\n");
     o("\t\t\t<tr>\n");
     o("\t\t\t\t<th>画像(ページの一番上に表示される画像です)</th>\n");
     o("\t\t\t\t<td><input type=\"file\" name=\"project.file\" /></td>\n");
     o("\t\t\t</tr>\n");
+    o("\t\t\t<tr>\n");
+    o("\t\t\t\t<th>ファイルアップロードの最大サイズ(Kb)</th>\n");
+    o("\t\t\t\t<td><input type=\"text\" name=\"project.upload_max_size\" value=\"%d\" /></td>\n", project_a->upload_max_size);
+    o("\t\t\t</tr>\n");
     o("\t\t</table>\n");
     o("\t\t<input class=\"button\" type=\"submit\" value=\"更新\" />\n");
     o("\t</form>\n");
@@ -199,12 +203,14 @@ void fill_upload_content_setting_file(SettingFile* sf)
 {
     int got_count = 0;
     char* buffer;
+    char file_name[DEFAULT_LENGTH];
     char b[DEFAULT_LENGTH];
     cgiFilePtr file;
-    strcpy(sf->name, "top_image");
+    string_set(sf->name, "top_image");
     cgiFormFileSize("project.file", &sf->size);
-    cgiFormFileName("project.file", sf->file_name, DEFAULT_LENGTH);
-    cgiFormFileContentType("project.file", sf->mime_type, DEFAULT_LENGTH);
+    cgiFormFileName("project.file", file_name, DEFAULT_LENGTH);
+    string_set(sf->file_name, file_name);
+    cgiFormFileContentType("project.file", string_rawstr(sf->mime_type), DEFAULT_LENGTH);
     buffer = sf->content = xalloc(sizeof(char) * sf->size);
     if (cgiFormFileOpen("project.file", &file) != cgiFormSuccess) {
         die("Could not open the file.");
@@ -229,12 +235,17 @@ void project_submit_action()
     Project* project_a = project_new();
     SettingFile* sf_a = setting_file_new();
     Database* db_a;
+    char name[DEFAULT_LENGTH];
+    char upload_max_size_str[DEFAULT_LENGTH];
     char buffer[DEFAULT_LENGTH];
 
     db_a = db_init(db_top_get_project_db_name(g_project_name, buffer));
     db_begin(db_a);
     project_a = db_get_project(db_a, project_a);
-    cgiFormStringNoNewlines("project.name", project_a->name, DEFAULT_LENGTH);
+    cgiFormStringNoNewlines("project.name", name, DEFAULT_LENGTH);
+    string_set(project_a->name, name);
+    cgiFormStringNoNewlines("project.upload_max_size", upload_max_size_str, DEFAULT_LENGTH);
+    project_a->upload_max_size = atoi(upload_max_size_str);
     db_update_project(db_a, project_a);
     project_free(project_a);
 
@@ -266,7 +277,7 @@ void items_action()
     project_a = db_get_project(db_a, project_a);
     output_header(project_a, "項目設定", "management.js", NAVI_ITEM);
 
-    o("<h2>%s 管理ツール</h2>", project_a->name);
+    o("<h2>%s 管理ツール</h2>", string_rawstr(project_a->name));
     o("<div id=\"top\">\n");
     project_free(project_a);
     o("<div id=\"setting_form\">\n"
@@ -275,13 +286,13 @@ void items_action()
       "\t\t<div class=\"description\">チケットID、投稿日時は編集できません。件名、投稿者、状態は、基本属性のため編集できる内容に制限があります。(削除不可、チケット属性、返信専用属性変更不可)</div>\n"
       "\t\t<div><a href=\"%s/%s/new_item\">新規項目の追加</a></div>\n", cgiScriptName, g_project_name_4_url);
 
-    list_alloc(element_types_a, ElementType);
+    list_alloc(element_types_a, ElementType, element_type_new, element_type_free);
     element_types_a = db_get_element_types_all(db_a, element_types_a);
     output_field_information_js();
     o("\t\t<ul id=\"field_list\">\n");
     foreach (it, element_types_a) {
         ElementType* et = it->element;
-        o("\t\t<li><a href=\"#field%d\">", et->id); h(et->name); o("</a></li>\n");
+        o("\t\t<li><a href=\"#field%d\">", et->id); hs(et->name); o("</a></li>\n");
     }
     o("\t\t</ul>\n"
       "\t\t<input class=\"button\" type=\"submit\" value=\"更新\" />\n");
@@ -289,17 +300,17 @@ void items_action()
         ElementType* et = it->element;
         List* items_a;
         Iterator* it;
-        list_alloc(items_a, ListItem);
+        list_alloc(items_a, ListItem, list_item_new, list_item_free);
         items_a = db_get_list_item(db_a, et->id, items_a);
         o("\t\t<a name=\"field%d\"></a>\n", et->id);
-        o("\t\t<h4>"); h(et->name); o("&nbsp;<a href=\"#top\">↑</a></h4>\n");
+        o("\t\t<h4>"); hs(et->name); o("&nbsp;<a href=\"#top\">↑</a></h4>\n");
         o("\t\t<input type=\"hidden\" name=\"field_ids\" value=\"%d\" />\n", et->id);
         o("\t\t<table class=\"item_table\" summary=\"item table\">\n"
           "\t\t\t<tr>\n"
           "\t\t\t\t<th class=\"required\">項目名<span class=\"required\">※</span></th>\n"
           "\t\t\t\t<td>\n"
           "\t\t\t\t\t<input class=\"required\" id=\"field%d.name\" type=\"text\" name=\"field%d.name\" ", et->id, et->id);
-        o(                  "value=\"");h(et->name);o("\" maxlength=\"1000\" />\n"
+        o(                  "value=\"");hs(et->name);o("\" maxlength=\"1000\" />\n"
           "\t\t\t\t\t<div class=\"description\">項目名です。</div>\n"
           "\t\t\t\t\t<div id=\"field%d.name.required\" class=\"error\"></div>\n", et->id);
         o("\t\t\t\t</td>\n"
@@ -308,7 +319,7 @@ void items_action()
           "\t\t\t\t<th>項目の説明文</th>\n"
           "\t\t\t\t<td>\n"
           "\t\t\t\t\t<input type=\"text\" name=\"field%d.description\" ", et->id);
-        o(                  "value=\"");h(et->description);o("\" maxlength=\"1000\" />\n"
+        o(                  "value=\"");hs(et->description);o("\" maxlength=\"1000\" />\n"
           "\t\t\t\t\t<div class=\"description\">項目の説明文です。投稿時に表示されます。</div>\n"
           "\t\t\t\t</td>\n"
           "\t\t\t</tr>\n"
@@ -393,36 +404,40 @@ void items_action()
                     o("\t\t\t\t\t\t<tr>\n"
                       "\t\t\t\t\t\t\t<td>\n"
                       "\t\t\t\t\t\t\t\t<input class=\"text\" type=\"text\" name=\"field%d.list_item%d.name\" ", et->id, item->id);
-                    o(                       "value=\"");h(item->name);o("\" maxlength=\"1000\" />\n"
+                    o(                       "value=\"");hs(item->name);o("\" maxlength=\"1000\" />\n"
                       "\t\t\t\t\t\t\t</td>\n"
-                      "\t\t\t\t\t\t\t<td>\n"
+                      "\t\t\t\t\t\t\t<td class=\"center\">\n"
                       "\t\t\t\t\t\t\t\t<input class=\"checkbox\" type=\"checkbox\" name=\"field%d.list_item%d.close\" value=\"1\" %s />\n", et->id, item->id, (item->close == 1) ? "checked=\"checked\"" : "");
                     o("\t\t\t\t\t\t\t</td>\n"
                       "\t\t\t\t\t\t\t<td>\n"
                       "\t\t\t\t\t\t\t\t<input class=\"number\" type=\"text\" name=\"field%d.list_item%d.sort\" value=\"%d\" maxlength=\"1000\" />\n", et->id, item->id, item->sort);
                     o("\t\t\t\t\t\t\t</td>\n"
-                      "\t\t\t\t\t\t\t<td>\n"
+                      "\t\t\t\t\t\t\t<td class=\"center\">\n"
                       "\t\t\t\t\t\t\t\t<input class=\"checkbox\" type=\"checkbox\" name=\"field%d.list_item%d.delete\" value=\"1\" />\n", et->id, item->id);
                     o("\t\t\t\t\t\t\t</td>\n"
                       "\t\t\t\t\t\t</tr>\n");
                 }
                 o("\t\t\t\t\t\t<tr>\n"
                   "\t\t\t\t\t\t\t<td>\n"
-                  "\t\t\t\t\t\t\t\t<input class=\"text\" type=\"text\" name=\"field%d.list_item_new.name\" value=\"\" maxlength=\"1000\" />\n", et->id);
+                  "\t\t\t\t\t\t\t\t<input class=\"text\" type=\"text\" name=\"field%d.list_item_new.name0\" value=\"\" maxlength=\"1000\" />\n", et->id);
+                o("\t\t\t\t\t\t\t</td>\n"
+                  "\t\t\t\t\t\t\t<td class=\"center\">\n"
+                  "\t\t\t\t\t\t\t\t<input class=\"checkbox\" type=\"checkbox\" name=\"field%d.list_item_new.close0\" value=\"\" />\n", et->id);
                 o("\t\t\t\t\t\t\t</td>\n"
                   "\t\t\t\t\t\t\t<td>\n"
-                  "\t\t\t\t\t\t\t\t<input class=\"checkbox\" type=\"checkbox\" name=\"field%d.list_item_new.close\" value=\"\" />\n", et->id);
+                  "\t\t\t\t\t\t\t\t<input class=\"number\" type=\"text\" name=\"field%d.list_item_new.sort0\" value=\"\" />\n", et->id);
                 o("\t\t\t\t\t\t\t</td>\n"
-                  "\t\t\t\t\t\t\t<td>\n"
-                  "\t\t\t\t\t\t\t\t<input class=\"number\" type=\"text\" name=\"field%d.list_item_new.sort\" value=\"\" />\n", et->id);
-                o("\t\t\t\t\t\t\t</td>\n"
-                  "\t\t\t\t\t\t\t<td>\n"
-                  "\t\t\t\t\t\t\t\t&nbsp;\n"
+                  "\t\t\t\t\t\t\t<td class=\"center\">\n"
+                  "\t\t\t\t\t\t\t\t<script type=\"text/javascript\">\n"
+                  "\t\t\t\t\t\t\t\t\t<!--\n"
+                  "\t\t\t\t\t\t\t\t\tdocument.write('<input type=\"button\" value=\"行追加\" id=\"add_new_row.field%d\" class=\"add_new_row\" />');\n"
+                  "\t\t\t\t\t\t\t\t\t//-->\n"
+                  "\t\t\t\t\t\t\t\t</script>\n"
                   "\t\t\t\t\t\t\t</td>\n"
                   "\t\t\t\t\t\t</tr>\n"
                   "\t\t\t\t\t</table>\n"
                   "\t\t\t\t\t<div class=\"description\">項目種別が選択式の場合の選択肢です。</div>\n"
-                  "\t\t\t\t\t<input id=\"field%d.auto_add_item\" class=\"checkbox\" type=\"checkbox\" name=\"field%d.auto_add_item\" ", et->id, et->id);
+                  "\t\t\t\t\t<input id=\"field%d.auto_add_item\" class=\"checkbox\" type=\"checkbox\" name=\"field%d.auto_add_item\" ", et->id, et->id, et->id);
                 o(                  "value=\"1\" %s />\n", et->auto_add_item == 1 ? "checked=\"checked\"" : "");
                 o("\t\t\t\t\t<label for=\"field%d.auto_add_item\">投稿時に、新規項目を指定可能とする。</label>\n", et->id);
                 o("\t\t\t\t\t<div class=\"description\">投稿時にテキストボックスを表示し、テキストボックスに入力された場合は、その項目を選択肢に追加する機能を付加するかどうかです。</div>\n"
@@ -434,9 +449,9 @@ void items_action()
           "\t\t\t\t<th>デフォルト値</th>\n"
           "\t\t\t\t<td>\n");
         if (et->type == ELEM_TYPE_TEXTAREA) {
-            o("\t\t\t\t\t<textarea name=\"field%d.default_value\" rows=\"2\" cols=\"10\" >", et->id);h(et->default_value);o("</textarea>\n");
+            o("\t\t\t\t\t<textarea name=\"field%d.default_value\" rows=\"2\" cols=\"10\" >", et->id);hs(et->default_value);o("</textarea>\n");
         } else {
-            o("\t\t\t\t\t<input class=\"text\" type=\"text\" name=\"field%d.default_value\" value=\"%s\" maxlength=\"1000\" />\n", et->id, et->default_value);
+            o("\t\t\t\t\t<input class=\"text\" type=\"text\" name=\"field%d.default_value\" value=\"%s\" maxlength=\"1000\" />\n", et->id, string_rawstr(et->default_value));
         }
         o("\t\t\t\t\t<div class=\"description\">投稿画面、返信画面での項目の初期値です。</div>\n"
           "\t\t\t\t</td>\n"
@@ -458,9 +473,27 @@ void items_action()
           "\t\t\t\t</td>\n"
           "\t\t\t</tr>\n"
           "\t\t</table>\n");
+        o("\t\t<table style=\"display:none\">\n"
+          "\t\t\t<tbody id=\"row_template\">\n"
+          "\t\t\t<tr>\n"
+          "\t\t\t\t<td>\n"
+          "\t\t\t\t\t<input class=\"text\" type=\"text\" name=\"field[id].list_item_new.name[no]\" value=\"\" maxlength=\"1000\" />\n"
+          "\t\t\t\t</td>\n"
+          "\t\t\t\t<td class=\"center\">\n"
+          "\t\t\t\t\t<input class=\"checkbox\" type=\"checkbox\" name=\"field[id].list_item_new.close[no]\" value=\"\" />\n"
+          "\t\t\t\t</td>\n"
+          "\t\t\t\t<td>\n"
+          "\t\t\t\t\t<input class=\"number\" type=\"text\" name=\"field[id].list_item_new.sort[no]\" value=\"\" />\n"
+          "\t\t\t\t</td>\n"
+          "\t\t\t\t<td class=\"center\">\n"
+          "\t\t\t\t\t&nbsp;\n"
+          "\t\t\t\t</td>\n"
+          "\t\t\t</tr>\n"
+          "\t\t\t</tbody>\n"
+          "\t\t</table>\n");
         if (et->id > BASIC_ELEMENT_MAX) {
             /* 基本項目は削除できないようにする。 */
-            o("\t\t<div class=\"delete_item\"><a href=\"%s/%s/delete_item/%d\">", cgiScriptName, g_project_name_4_url, et->id);o("この項目(");h(et->name);o(")の削除</a></div>\n");
+            o("\t\t<div class=\"delete_item\"><a href=\"%s/%s/delete_item/%d\">", cgiScriptName, g_project_name_4_url, et->id);o("この項目(");hs(et->name);o(")の削除</a></div>\n");
         }
         list_free(items_a);
         o("\t\t<input class=\"button\" type=\"submit\" value=\"更新\" />\n");
@@ -492,26 +525,28 @@ void update_elements(Database* db)
 {
     char** ids;
     int i = -1;
+
     if ((cgiFormStringMultiple("field_ids", &ids)) == cgiFormNotFound) {
         die("cannot find field_ids.");
     }
     while (ids[++i]) {
+        int j;
         char* id = ids[i];
         char name[DEFAULT_LENGTH];
         char value[DEFAULT_LENGTH];
         List* items_a;
         Iterator* it;
-        ElementType* et_a = xalloc(sizeof(ElementType));
+        ElementType* et_a = element_type_new();
 
         et_a = db_get_element_type(db, atoi(id), et_a);
 
         sprintf(name, "field%s.name", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        strcpy(et_a->name, value);
+        string_set(et_a->name, value);
 
         sprintf(name, "field%s.description", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-        strcpy(et_a->description, value);
+        string_set(et_a->description, value);
 
         sprintf(name, "field%s.ticket_property", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
@@ -535,7 +570,7 @@ void update_elements(Database* db)
 
         sprintf(name, "field%s.default_value", id);
         cgiFormString(name, value, DEFAULT_LENGTH);
-        strcpy(et_a->default_value, value);
+        string_set(et_a->default_value, value);
 
         sprintf(name, "field%s.auto_add_item", id);
         cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
@@ -548,7 +583,7 @@ void update_elements(Database* db)
             case ELEM_TYPE_LIST_SINGLE:
             case ELEM_TYPE_LIST_MULTI:
                 /* 選択要素のあるelementだけ、list_itemの更新を行なう。 */
-                list_alloc(items_a, ListItem);
+                list_alloc(items_a, ListItem, list_item_new, list_item_free);
                 items_a = db_get_list_item(db, et_a->id, items_a);
                 foreach (it, items_a) {
                     ListItem* item = it->element;
@@ -556,7 +591,7 @@ void update_elements(Database* db)
                     strcpy(value, "");
                     sprintf(name, "field%d.list_item%d.name", et_a->id, item->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                    strcpy(item->name, value);
+                    string_set(item->name, value);
                     sprintf(name, "field%d.list_item%d.close", et_a->id, item->id);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
                     item->close = atoi(value);
@@ -571,27 +606,34 @@ void update_elements(Database* db)
                         db_update_list_item(db, item);
                 }
                 list_free(items_a);
-                strcpy(name, "");
-                strcpy(value, "");
-                sprintf(name, "field%d.list_item_new.name", et_a->id);
-                cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                if (strlen(value) > 0) {
-                    ListItem* item_a = xalloc(sizeof(ListItem));
-                    item_a->element_type_id = et_a->id;
-                    strcpy(item_a->name, value);
-                    sprintf(name, "field%d.list_item_new.close", et_a->id);
+                //新しい項目の追加
+                for (j = 0; j < 100; j++) {
+                    strcpy(name, "");
+                    strcpy(value, "");
+                    sprintf(name, "field%d.list_item_new.name%d", et_a->id, j);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                    item_a->close = atoi(value);
-                    sprintf(name, "field%d.list_item_new.sort", et_a->id);
-                    cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
-                    item_a->sort = atoi(value);
-                    db_register_list_item(db, item_a);
-                    xfree(item_a);
+                    if (strlen(value) > 0) {
+                        ListItem* item_a = list_item_new();
+                        item_a->element_type_id = et_a->id;
+                        d("%s\n", value);
+                        string_set(item_a->name, value);
+                        sprintf(name, "field%d.list_item_new.close%d", et_a->id, j);
+                        cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
+                        item_a->close = atoi(value);
+                        sprintf(name, "field%d.list_item_new.sort%d", et_a->id, j);
+                        cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
+                        item_a->sort = atoi(value);
+                        db_register_list_item(db, item_a);
+                        xfree(item_a);
+                    } else {
+                        //名称が取得できない場合は、追加終了
+                        break;
+                    }
                 }
                 break;
         }
 
-        xfree(et_a);
+        element_type_free(et_a);
     }
     cgiStringArrayFree(ids);
 }
@@ -606,7 +648,7 @@ void new_item_action()
     project_a = db_get_project(db_a, project_a);
     output_header(project_a, "新規項目登録", "new_item.js", NAVI_OTHER);
 
-    o("<h2>%s 管理ツール</h2>", project_a->name);
+    o("<h2>%s 管理ツール</h2>", string_rawstr(project_a->name));
     project_free(project_a);
     o(      "<div id=\"new_item\">\n"
             "<h3>項目の追加</h3>\n"
@@ -740,7 +782,7 @@ void new_item_action()
 }
 void new_item_submit_action()
 {
-    ElementType* et_a = xalloc(sizeof(ElementType));
+    ElementType* et_a = element_type_new();
     char value[DEFAULT_LENGTH];
     int i, e_type_id;
     Database* db_a;
@@ -749,9 +791,11 @@ void new_item_submit_action()
     db_a = db_init(db_top_get_project_db_name(g_project_name, buffer));
     db_begin(db_a);
 
-    cgiFormStringNoNewlines("field.name", et_a->name, DEFAULT_LENGTH);
+    cgiFormStringNoNewlines("field.name", value, DEFAULT_LENGTH);
+    string_set(et_a->name, value);
 
-    cgiFormStringNoNewlines("field.description", et_a->description, DEFAULT_LENGTH);
+    cgiFormStringNoNewlines("field.description", value, DEFAULT_LENGTH);
+    string_set(et_a->description, value);
 
     cgiFormStringNoNewlines("field.type", value, DEFAULT_LENGTH);
     et_a->type = atoi(value);
@@ -774,11 +818,12 @@ void new_item_submit_action()
         case ELEM_TYPE_LIST_MULTI:
             for (i = 0; i < ADD_ITEM_COUNT; i++) {
                 char name[DEFAULT_LENGTH];
-                ListItem* item_a = xalloc(sizeof(ListItem));
+                ListItem* item_a = list_item_new();
                 item_a->element_type_id = e_type_id;
                 sprintf(name, "field.list_item_new%d.name", i);
-                cgiFormStringNoNewlines(name, item_a->name, DEFAULT_LENGTH);
-                if (strlen(item_a->name) > 0) {
+                cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
+                string_set(item_a->name, value);
+                if (string_len(item_a->name) > 0) {
                     sprintf(name, "field.list_item_new%d.close", i);
                     cgiFormStringNoNewlines(name, value, DEFAULT_LENGTH);
                     item_a->close = atoi(value);
@@ -791,7 +836,7 @@ void new_item_submit_action()
             }
             break;
     }
-    xfree(et_a);
+    element_type_free(et_a);
     db_commit(db_a);
     db_finish(db_a);
     redirect("", "追加しました");
@@ -801,7 +846,7 @@ void delete_item_action()
     char* e_type_id;
     int iid;
     Project* project_a = project_new();
-    ElementType* et_a = xalloc(sizeof(ElementType));
+    ElementType* et_a = element_type_new();
     Database* db_a;
     char buffer[DEFAULT_LENGTH];
 
@@ -814,10 +859,10 @@ void delete_item_action()
     output_header(project_a, "項目削除", "delete_item.js", NAVI_OTHER);
 
     et_a = db_get_element_type(db_a, iid, et_a);
-    o("<h2>%s 管理ツール</h2>", project_a->name);
+    o("<h2>%s 管理ツール</h2>", string_rawstr(project_a->name));
     project_free(project_a);
     o(      "<div id=\"delete_item/%d\">\n", iid);
-    o(      "<h3>項目(");h(et_a->name);o(")の削除</h3>\n"
+    o(      "<h3>項目(");hs(et_a->name);o(")の削除</h3>\n"
             "<form id=\"delete_item_form\" action=\"%s/%s/delete_item_submit/%d\" method=\"post\">\n"
             "<div class=\"infomation\"><strong>削除すると元には戻せません。"
             "登録されているチケットの項目についても参照できなくなります。</strong></div>"
@@ -827,7 +872,7 @@ void delete_item_action()
     o(      "</div>\n");
     db_finish(db_a);
     output_footer();
-    xfree(et_a);
+    element_type_free(et_a);
 }
 void delete_item_submit_action()
 {
@@ -860,7 +905,7 @@ void style_action()
     db_a = db_init(db_top_get_project_db_name(g_project_name, buffer));
     project_a = db_get_project(db_a, project_a);
     output_header(project_a, "スタイル設定", "style.js", NAVI_STYLE);
-    o(      "<h2>%s 管理ツール</h2>", project_a->name);
+    o(      "<h2>%s 管理ツール</h2>", string_rawstr(project_a->name));
     project_free(project_a);
     o(      "<div id=\"top\">\n"
             "<h3>スタイルシートの編集</h3>\n"
@@ -883,23 +928,23 @@ void style_action()
         Iterator* it;
         o(      "<div class=\"description\">以下は、一覧の背景設定用のサンプルです。</div>\n");
         o(      "<pre>\n");
-        list_alloc(element_types_a, ElementType);
+        list_alloc(element_types_a, ElementType, element_type_new, element_type_free);
         element_types_a = db_get_element_types_all(db_a, element_types_a);
         foreach (it, element_types_a) {
             ElementType* et = it->element;
             if (et->type == ELEM_TYPE_LIST_SINGLE && et->display_in_list) {
                 List* items_a;
                 Iterator* it_item;
-                list_alloc(items_a, ListItem);
+                list_alloc(items_a, ListItem, list_item_new, list_item_free);
                 items_a = db_get_list_item(db_a, et->id, items_a);
                 o(  "/* ================================ */ \n"
-                    "/* チケット一覧の"); h(et->name); o("の背景色設定     */\n");
+                    "/* チケット一覧の"); hs(et->name); o("の背景色設定     */\n");
                 o(  "/* ================================ */ \n");
                 foreach (it_item, items_a) {
                     ListItem* item = it_item->element;
-                    o("/* "); h(item->name); o(" */\n");
+                    o("/* "); hs(item->name); o(" */\n");
                     o("#ticket_list td.field%d-", et->id);
-                    css_field(item->name);
+                    css_field(string_rawstr(item->name));
                     o(" {\n");
                     o(" background-color: lightyellow !important;\n");
                     o("}\n");
@@ -919,13 +964,13 @@ void style_submit_action()
     SettingFile* sf_a = setting_file_new();
     Database* db_a;
     char buffer[DEFAULT_LENGTH];
-    strcpy(sf_a->name, "user.css");
-    strcpy(sf_a->file_name, "");
+    string_set(sf_a->name, "user.css");
+    string_set(sf_a->file_name, "");
     sf_a->content = xalloc(sizeof(char) * VALUE_LENGTH);
 
     cgiFormString("edit_css", sf_a->content, VALUE_LENGTH);
     sf_a->size = strlen(sf_a->content);
-    strcpy(sf_a->mime_type, "text/css");
+    string_set(sf_a->mime_type, "text/css");
     db_a = db_init(db_top_get_project_db_name(g_project_name, buffer));
     db_setting_file_save(db_a, sf_a);
     db_finish(db_a);
@@ -941,7 +986,7 @@ void admin_help_action()
     db_a = db_init(db_top_get_project_db_name(g_project_name, buffer));
     project_a = db_get_project(db_a, project_a);
     output_header(project_a, "ヘルプ", NULL, NAVI_ADMIN_HELP);
-    o(      "<h2>");h(project_a->name);o("</h2>\n"
+    o(      "<h2>");h(string_rawstr(project_a->name));o("</h2>\n"
             "<div id=\"top\">\n");
     wiki_out(db_a, "adminhelp");
     o(      "</div>\n");
