@@ -35,13 +35,13 @@ static List* db_get_element_types(Database* db, bool all, List* element_types)
         et->ticket_property = sqlite3_column_int(stmt, 2);
         et->reply_property = sqlite3_column_int(stmt, 3);
         et->required = sqlite3_column_int(stmt, 4);
-        strcpy(et->name, (char*)sqlite3_column_text(stmt, 5));
-        strcpy(et->description, (char*)sqlite3_column_text(stmt, 6));
+        string_set(et->name, (char*)sqlite3_column_text(stmt, 5));
+        string_set(et->description, (char*)sqlite3_column_text(stmt, 6));
         et->display_in_list = sqlite3_column_int(stmt, 7);
         et->sort = sqlite3_column_int(stmt, 8);
         value = sqlite3_column_text(stmt, 9);
         if (value != NULL) {
-            strcpy(et->default_value, (char*)value);
+            string_set(et->default_value, (char*)value);
         }
         et->auto_add_item = sqlite3_column_int(stmt, 10);
         list_add(element_types, et);
@@ -85,12 +85,12 @@ ElementType* db_get_element_type(Database* db, int id, ElementType* e)
         e->ticket_property = sqlite3_column_int(stmt, 2);
         e->reply_property = sqlite3_column_int(stmt, 3);
         e->required = sqlite3_column_int(stmt, 4);
-        strcpy(e->name, (char*)sqlite3_column_text(stmt, 5));
-        strcpy(e->description, (char*)sqlite3_column_text(stmt, 6));
+        string_set(e->name, (char*)sqlite3_column_text(stmt, 5));
+        string_set(e->description, (char*)sqlite3_column_text(stmt, 6));
         e->auto_add_item = sqlite3_column_int(stmt, 7);
         value = sqlite3_column_text(stmt, 8);
         if (value != NULL)
-            strcpy(e->default_value, (char*)value);
+            string_set(e->default_value, (char*)value);
         e->display_in_list = sqlite3_column_int(stmt, 9);
         e->sort = sqlite3_column_int(stmt, 10);
         break;
@@ -115,7 +115,7 @@ List* db_get_list_item(Database* db, const int element_type, List* items)
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         ListItem* item = list_new_element(items);
         item->id = sqlite3_column_int(stmt, 0);
-        strcpy(item->name, (char*)sqlite3_column_text(stmt, 1));
+        string_set(item->name, (char*)sqlite3_column_text(stmt, 1));
         item->close = sqlite3_column_int(stmt, 2);
         item->sort = sqlite3_column_int(stmt, 3);
         list_add(items, item);
@@ -180,7 +180,7 @@ int db_register_ticket(Database* db, Message* ticket)
                 "select close from list_item "
                 "where list_item.element_type_id = ? and list_item.name = ?",
                 COLUMN_TYPE_INT, e->element_type_id,
-                COLUMN_TYPE_TEXT, e->str_val,
+                COLUMN_TYPE_TEXT, string_rawstr(e->str_val),
                 COLUMN_TYPE_END);
         if (c != INVALID_INT && c != 0) {
             closed = true;
@@ -206,7 +206,7 @@ int db_register_ticket(Database* db, Message* ticket)
     sqlite3_bind_text(stmt, i++, registerdate, strlen(registerdate), NULL);
     foreach (it, elements) {
         Element* e = it->element;
-        sqlite3_bind_text(stmt, i++, e->str_val, strlen(e->str_val), NULL);
+        sqlite3_bind_text(stmt, i++, string_rawstr(e->str_val), string_len(e->str_val), NULL);
     }
     if (exec_and_wait_4_done(stmt) != SQLITE_RETURN_OK) goto error;
     message_id = sqlite3_last_insert_rowid(db->handle);
@@ -354,7 +354,7 @@ static String* get_search_sql_string(Database* db, List* conditions, Condition*
             char name[DEFAULT_LENGTH];
             char val[DEFAULT_LENGTH];
             if (cond->element_type_id > 0) continue;
-            if (strlen(cond->value) == 0) continue;
+            if (string_len(cond->value) == 0) continue;
             if (i++) string_append(sql_string, " and ");
             switch (cond->element_type_id) {
                 case ELEM_ID_REGISTERDATE:
@@ -378,7 +378,7 @@ static String* get_search_sql_string(Database* db, List* conditions, Condition*
     if (keywords->size > 0) {
         String* columns_a = string_new(0);
         List* element_types_a;
-        list_alloc(element_types_a, ElementType);
+        list_alloc(element_types_a, ElementType, element_type_new, element_type_free);
         element_types_a = db_get_element_types_all(db, element_types_a);
         columns_a = create_columns_like_exp(element_types_a, "m_all", keywords, columns_a);
         if (valid_condition_size(conditions))
@@ -393,7 +393,7 @@ static String* get_search_sql_string(Database* db, List* conditions, Condition*
     if (sort != NULL && sort->element_type_id != 0) {
         char column[DEFAULT_LENGTH];
         char sort_type[DEFAULT_LENGTH];
-        sprintf(sort_type, "%s", strstr(sort->value, "reverse") ? "desc" : "asc");
+        sprintf(sort_type, "%s", strstr(string_rawstr(sort->value), "reverse") ? "desc" : "asc");
         switch (sort->element_type_id) {
             case -1:
                 sprintf(column, "t.id %s, ", sort_type);
@@ -433,7 +433,7 @@ int set_conditions(Database* db, sqlite3_stmt* stmt, List* conditions, List* key
         List* element_types_a;
         Iterator* it_keyword;
         Iterator* it;
-        list_alloc(element_types_a, ElementType);
+        list_alloc(element_types_a, ElementType, element_type_new, element_type_free);
         element_types_a = db_get_element_types_all(db, element_types_a);
         foreach (it_keyword, keywords) {
             char* word = it_keyword->element;
@@ -454,11 +454,11 @@ SearchResult* db_get_tickets_by_status(Database* db, const char* status, SearchR
     sqlite3_stmt *stmt = NULL;
     List* keywords_a;
 
-    list_alloc(keywords_a, char);
-    list_alloc(conditions, Condition);
+    list_alloc(keywords_a, char, NULL, NULL);
+    list_alloc(conditions, Condition, condition_new, condition_free);
     cond_status = list_new_element(conditions);
     cond_status->element_type_id = ELEM_ID_STATUS;
-    strcpy(cond_status->value, status);
+    string_set(cond_status->value, status);
     list_add(conditions, cond_status);
     sql_a = get_search_sql_string(db, conditions, NULL, keywords_a, sql_a);
     string_append(sql_a, " limit ? ");
@@ -495,7 +495,7 @@ SearchResult* db_search_tickets(Database* db, List* conditions, char* q, Conditi
     sqlite3_stmt *stmt = NULL;
     List* keywords_a;
 
-    list_alloc(keywords_a, char);
+    list_alloc(keywords_a, char, NULL, NULL);
     keywords_a = parse_keywords(keywords_a, q);
     sql_a = get_search_sql_string(db, conditions, sorts, keywords_a, sql_a);
 
@@ -532,7 +532,7 @@ SearchResult* db_search_tickets(Database* db, List* conditions, char* q, Conditi
         while (SQLITE_ROW == (r = sqlite3_step(stmt))){
             State* s = list_new_element(result->states);
             result->hit_count += s->count = sqlite3_column_int(stmt, 0);
-            strcpy(s->name, (char*)sqlite3_column_text(stmt, 1));
+            string_set(s->name, (char*)sqlite3_column_text(stmt, 1));
             list_add(result->states, s);
         }
         string_free(s);
@@ -554,7 +554,7 @@ SearchResult* db_search_tickets_4_report(Database* db, List* conditions, char* q
     sqlite3_stmt *stmt = NULL;
     List* keywords_a;
 
-    list_alloc(keywords_a, char);
+    list_alloc(keywords_a, char, NULL, NULL);
     keywords_a = parse_keywords(keywords_a, q);
     sql_a = get_search_sql_string(db, conditions, sorts, keywords_a, sql_a);
 
@@ -592,8 +592,7 @@ void create_columns_exp(List* element_types, char* table_name, char* buf)
 static void set_str_val(Element* e, const unsigned char* str_val)
 {
     if (str_val != NULL) {
-        e->str_val = xalloc(sizeof(char) * strlen((char*)str_val) + 1);
-        strcpy(e->str_val, (char*)str_val);
+        string_set(e->str_val, (const char*)str_val);
     }
 }
 List* db_get_last_elements_4_list(Database* db, const int ticket_id, List* elements)
@@ -606,7 +605,7 @@ List* db_get_last_elements_4_list(Database* db, const int ticket_id, List* eleme
     List* element_types_a = NULL;
     Iterator* it;
 
-    list_alloc(element_types_a, ElementType);
+    list_alloc(element_types_a, ElementType, element_type_new, element_type_free);
     element_types_a = db_get_element_types_4_list(db, element_types_a);
     create_columns_exp(element_types_a, "last_m", columns);
     sprintf(sql, "select t.id, org_m.field%d ", ELEM_ID_SENDER);
@@ -676,7 +675,7 @@ List* db_get_last_elements(Database* db, const int ticket_id, List* elements)
     List* element_types_a = NULL;
     char columns[DEFAULT_LENGTH] = "";
 
-    list_alloc(element_types_a, ElementType);
+    list_alloc(element_types_a, ElementType, element_type_new, element_type_free);
     element_types_a = db_get_element_types_all(db, element_types_a);
     create_columns_exp(element_types_a, "m", columns);
     strcpy(sql, "select m.id");
@@ -741,7 +740,7 @@ List* db_get_elements(Database* db, int message_id, List* elements)
     char columns[DEFAULT_LENGTH] = "";
     List* element_types_a = NULL;
 
-    list_alloc(element_types_a, ElementType);
+    list_alloc(element_types_a, ElementType, element_type_new, element_type_free);
     element_types_a = db_get_element_types_all(db, element_types_a);
     create_columns_exp(element_types_a, "m", columns);
 
@@ -827,12 +826,16 @@ Project* db_get_project(Database* db, Project* project)
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         char* name = (char*)sqlite3_column_text(stmt, 0);
         if (strcmp(name, "project_name") == 0)
-            strcpy(project->name, (char*)sqlite3_column_text(stmt, 1));
+            string_set(project->name, (char*)sqlite3_column_text(stmt, 1));
         else if (strcmp(name, "home_description") == 0)
-            strcpy(project->home_description, (char*)sqlite3_column_text(stmt, 1));
+            string_set(project->home_description, (char*)sqlite3_column_text(stmt, 1));
         else if (strcmp(name, "home_url") == 0)
-            strcpy(project->home_url, (char*)sqlite3_column_text(stmt, 1));
+            string_set(project->home_url, (char*)sqlite3_column_text(stmt, 1));
+        else if (strcmp(name, "upload_max_size") == 0)
+            project->upload_max_size = sqlite3_column_int(stmt, 1);
     }
+    /* アップロードファイルサイズの上限値が有効な値でない場合は、512kbを設定する。 */
+    project->upload_max_size = (project->upload_max_size <= 0) ? 512 : project->upload_max_size;
 
     sqlite3_finalize(stmt);
 
@@ -846,7 +849,7 @@ void db_update_project(Database* db, Project* project)
             "update setting set "
             "value = ? "
             "where name = 'project_name'",
-            COLUMN_TYPE_TEXT, project->name,
+            COLUMN_TYPE_TEXT, string_rawstr(project->name),
             COLUMN_TYPE_END) != 1)
         die("no seting to update? or too many?");
     if (exec_query(
@@ -854,7 +857,7 @@ void db_update_project(Database* db, Project* project)
             "update setting set "
             "value = ? "
             "where name = 'home_description'",
-            COLUMN_TYPE_TEXT, project->home_description,
+            COLUMN_TYPE_TEXT, string_rawstr(project->home_description),
             COLUMN_TYPE_END) != 1)
         die("no seting to update? or too many?");
     if (exec_query(
@@ -862,7 +865,15 @@ void db_update_project(Database* db, Project* project)
             "update setting set "
             "value = ? "
             "where name = 'home_url'",
-            COLUMN_TYPE_TEXT, project->home_url,
+            COLUMN_TYPE_TEXT, string_rawstr(project->home_url),
+            COLUMN_TYPE_END) != 1)
+        die("no seting to update? or too many?");
+    if (exec_query(
+            db,
+            "update setting set "
+            "value = ? "
+            "where name = 'upload_max_size'",
+            COLUMN_TYPE_INT, project->upload_max_size,
             COLUMN_TYPE_END) != 1)
         die("no seting to update? or too many?");
 }
@@ -899,11 +910,11 @@ void db_update_element_type(Database* db, ElementType* et)
             COLUMN_TYPE_INT, et->ticket_property,
             COLUMN_TYPE_INT, et->reply_property,
             COLUMN_TYPE_INT, et->required,
-            COLUMN_TYPE_TEXT, et->name,
-            COLUMN_TYPE_TEXT, et->description,
+            COLUMN_TYPE_TEXT, string_rawstr(et->name),
+            COLUMN_TYPE_TEXT, string_rawstr(et->description),
             COLUMN_TYPE_INT, et->sort,
             COLUMN_TYPE_INT, et->display_in_list,
-            COLUMN_TYPE_TEXT, et->default_value,
+            COLUMN_TYPE_TEXT, string_rawstr(et->default_value),
             COLUMN_TYPE_INT, et->auto_add_item,
             COLUMN_TYPE_INT, et->id,
             COLUMN_TYPE_END) != 1)
@@ -915,7 +926,7 @@ void db_update_list_item(Database* db, ListItem* item)
             db,
             "update list_item set "
             "name = ?, close = ?, sort = ? where id = ?",
-            COLUMN_TYPE_TEXT, item->name,
+            COLUMN_TYPE_TEXT, string_rawstr(item->name),
             COLUMN_TYPE_INT, item->close,
             COLUMN_TYPE_INT, item->sort,
             COLUMN_TYPE_INT, item->id,
@@ -938,7 +949,7 @@ void db_register_list_item(Database* db, ListItem* item)
             "insert into list_item (id, element_type_id, name, close, sort)"
             "values (NULL, ?, ?, ?, ?)",
             COLUMN_TYPE_INT, item->element_type_id,
-            COLUMN_TYPE_TEXT, item->name,
+            COLUMN_TYPE_TEXT, string_rawstr(item->name),
             COLUMN_TYPE_INT, item->close,
             COLUMN_TYPE_INT, item->sort,
             COLUMN_TYPE_END);
@@ -956,8 +967,8 @@ int db_register_element_type(Database* db, ElementType* et)
             COLUMN_TYPE_INT, et->ticket_property,
             COLUMN_TYPE_INT, et->reply_property,
             COLUMN_TYPE_INT, et->required,
-            COLUMN_TYPE_TEXT, et->name,
-            COLUMN_TYPE_TEXT, et->description,
+            COLUMN_TYPE_TEXT, string_rawstr(et->name),
+            COLUMN_TYPE_TEXT, string_rawstr(et->description),
             COLUMN_TYPE_INT, et->auto_add_item,
             COLUMN_TYPE_INT, et->display_in_list,
             COLUMN_TYPE_INT, et->sort,
@@ -1002,7 +1013,7 @@ List* db_get_states_has_not_close(Database* db, List* states)
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         State* s = list_new_element(states);
         s->id = sqlite3_column_int(stmt, 0);
-        strcpy(s->name, (char*)sqlite3_column_text(stmt, 1));
+        string_set(s->name, (char*)sqlite3_column_text(stmt, 1));
         s->count = sqlite3_column_int(stmt, 2);
         list_add(states, s);
     }
@@ -1035,7 +1046,7 @@ List* db_get_states(Database* db, List* states)
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         State* s = list_new_element(states);
-        strcpy(s->name, (char*)sqlite3_column_text(stmt, 0));
+        string_set(s->name, (char*)sqlite3_column_text(stmt, 0));
         s->count = sqlite3_column_int(stmt, 1);
         list_add(states, s);
     }
@@ -1069,7 +1080,7 @@ List* db_get_statictics_multi(Database* db, List* states, const int element_type
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         State* s = list_new_element(states);
         s->id = sqlite3_column_int(stmt, 0);
-        strcpy(s->name, (char*)sqlite3_column_text(stmt, 1));
+        string_set(s->name, (char*)sqlite3_column_text(stmt, 1));
         s->count = sqlite3_column_int(stmt, 2);
         list_add(states, s);
     }
@@ -1103,7 +1114,7 @@ List* db_get_statictics(Database* db, List* states, const int element_type_id)
     while (SQLITE_ROW == (r = sqlite3_step(stmt))){
         State* s = list_new_element(states);
         s->id = sqlite3_column_int(stmt, 0);
-        strcpy(s->name, (char*)sqlite3_column_text(stmt, 1));
+        string_set(s->name, (char*)sqlite3_column_text(stmt, 1));
         s->count = sqlite3_column_int(stmt, 2);
         list_add(states, s);
     }
@@ -1135,9 +1146,9 @@ ElementFile* db_get_element_file(Database* db, int id, ElementFile* file)
         char* p_dist;
         file->id = sqlite3_column_int(stmt, 0);
         file->element_type_id = sqlite3_column_int(stmt, 1);
-        strcpy(file->name, (char*)sqlite3_column_text(stmt, 2));
+        string_set(file->name, (char*)sqlite3_column_text(stmt, 2));
         file->size = sqlite3_column_int(stmt, 3);
-        strcpy(file->mime_type, (char*)sqlite3_column_text(stmt, 4));
+        string_set(file->mime_type, (char*)sqlite3_column_text(stmt, 4));
         len = sqlite3_column_bytes(stmt, 5);
         p_dist = file->content = xalloc(sizeof(char) * len);
         p_src = (char*)sqlite3_column_blob(stmt, 5);
@@ -1224,7 +1235,7 @@ void db_register_wiki(Database* db, Wiki* wiki)
             db,
             "insert into wiki (id, name, content, registerdate)"
             "values (NULL, ?, ?, ?)",
-            COLUMN_TYPE_TEXT, wiki->name,
+            COLUMN_TYPE_TEXT, string_rawstr(wiki->name),
             COLUMN_TYPE_TEXT, wiki->content,
             COLUMN_TYPE_TEXT, registerdate,
             COLUMN_TYPE_END);
@@ -1249,7 +1260,7 @@ Wiki* db_get_newest_wiki(Database* db, char* page_name, Wiki* wiki)
     while (SQLITE_ROW == (r = sqlite3_step(stmt))) {
         const unsigned char* value;
         wiki->id = sqlite3_column_int(stmt, 0);
-        strcpy(wiki->name, (char*)sqlite3_column_text(stmt, 1));
+        string_set(wiki->name, (char*)sqlite3_column_text(stmt, 1));
         value = sqlite3_column_text(stmt, 2);
         wiki->content = xalloc(sizeof(char) * strlen((char*)value) + 1);
         strcpy(wiki->content, (char*)value);
@@ -1272,11 +1283,11 @@ void db_setting_file_save(Database* db, SettingFile* sf)
                 " mime_type = ?, "
                 " content = ? "
                 "where name = ?",
-                COLUMN_TYPE_TEXT, sf->file_name,
+                COLUMN_TYPE_TEXT, string_rawstr(sf->file_name),
                 COLUMN_TYPE_INT, sf->size,
-                COLUMN_TYPE_TEXT, sf->mime_type,
+                COLUMN_TYPE_TEXT, string_rawstr(sf->mime_type),
                 COLUMN_TYPE_TEXT, sf->content,
-                COLUMN_TYPE_TEXT, sf->name,
+                COLUMN_TYPE_TEXT, string_rawstr(sf->name),
                 COLUMN_TYPE_END) != 1)
         die("failed to update setting file.");
 }
@@ -1298,10 +1309,10 @@ SettingFile* db_get_setting_file(Database* db, char* name, SettingFile* file)
         char* p_src;
         char* p_dist;
         if (sqlite3_column_text(stmt, 0)) 
-            strcpy(file->name, (char*)sqlite3_column_text(stmt, 0));
+            string_set(file->name, (char*)sqlite3_column_text(stmt, 0));
         file->size = sqlite3_column_int(stmt, 1);
         if (sqlite3_column_text(stmt, 2)) 
-            strcpy(file->mime_type, (char*)sqlite3_column_text(stmt, 2));
+            string_set(file->mime_type, (char*)sqlite3_column_text(stmt, 2));
         len = sqlite3_column_bytes(stmt, 3);
         p_dist = file->content = xalloc(sizeof(char) * len);
         p_src = (char*)sqlite3_column_blob(stmt, 3);
@@ -1329,9 +1340,9 @@ void db_update_top_image(Database* db, SettingFile* sf)
                 " mime_type = ?, "
                 " content = ? "
                 "where name = 'top_image'",
-                COLUMN_TYPE_TEXT, sf->file_name,
+                COLUMN_TYPE_TEXT, string_rawstr(sf->file_name),
                 COLUMN_TYPE_INT, sf->size,
-                COLUMN_TYPE_TEXT, sf->mime_type,
+                COLUMN_TYPE_TEXT, string_rawstr(sf->mime_type),
                 COLUMN_TYPE_BLOB_SETTING_FILE, sf,
                 COLUMN_TYPE_END) != 1)
         die("update failed.");
