@@ -24,13 +24,14 @@ List* db_top_get_all_project_infos(Database* db, List* project_infos)
     sqlite3_reset(stmt);
 
     while (SQLITE_ROW == (r = sqlite3_step(stmt))) {
-        ProjectInfo* project_info = list_new_element(project_infos);
+        ProjectInfo* pi = list_new_element(project_infos);
 
-        project_info->id = sqlite3_column_int(stmt, 0);
-        strncpy(project_info->name, (char*)sqlite3_column_text(stmt, 1), DEFAULT_LENGTH - 1);
-        project_info->sort = sqlite3_column_int(stmt, 2);
-        project_info->deleted = sqlite3_column_int(stmt, 3);
-        list_add(project_infos, project_info);
+        pi->id = sqlite3_column_int(stmt, 0);
+/*    d("db_top_get_all_project_infos 1 [%x]\n", pi->name);*/
+        string_set(pi->name, (char*)sqlite3_column_text(stmt, 1));
+        pi->sort = sqlite3_column_int(stmt, 2);
+        pi->deleted = sqlite3_column_int(stmt, 3);
+        list_add(project_infos, pi);
     }
     if (SQLITE_DONE != r)
         goto error;
@@ -53,7 +54,7 @@ ProjectInfo* db_top_get_project_info(Database* db, ProjectInfo* project_info, ch
 
     while (SQLITE_ROW == sqlite3_step(stmt)) {
         project_info->id = sqlite3_column_int(stmt, 0);
-        strncpy(project_info->name, (char*)sqlite3_column_text(stmt, 1), DEFAULT_LENGTH - 1);
+        string_set(project_info->name, (char*)sqlite3_column_text(stmt, 1));
         project_info->sort = sqlite3_column_int(stmt, 2);
         project_info->deleted = sqlite3_column_int(stmt, 3);
         break;
@@ -67,13 +68,13 @@ ERROR_LABEL(db->handle)
 void db_top_update_project(Database* db, Project* project)
 {
     exec_query(db, "update setting set value = ? where name = 'project_name'",
-            COLUMN_TYPE_TEXT, project->name,
+            COLUMN_TYPE_TEXT, string_rawstr(project->name),
             COLUMN_TYPE_END);
     exec_query(db, "update setting set value = ? where name = 'home_description'",
-            COLUMN_TYPE_TEXT, project->home_description,
+            COLUMN_TYPE_TEXT, string_rawstr(project->home_description),
             COLUMN_TYPE_END);
     exec_query(db, "update setting set value = ? where name = 'home_url'",
-            COLUMN_TYPE_TEXT, project->home_url,
+            COLUMN_TYPE_TEXT, string_rawstr(project->home_url),
             COLUMN_TYPE_END);
 }
 void db_top_update_project_infos(Database* db, List* project_infos)
@@ -83,7 +84,7 @@ void db_top_update_project_infos(Database* db, List* project_infos)
     foreach (it, project_infos) {
         ProjectInfo* pi = it->element;
         exec_query(db, "update project_info set name = ?, sort = ? , deleted = ? where id = ?",
-                COLUMN_TYPE_TEXT, pi->name,
+                COLUMN_TYPE_TEXT, string_rawstr(pi->name),
                 COLUMN_TYPE_INT, pi->sort,
                 COLUMN_TYPE_INT, pi->deleted,
                 COLUMN_TYPE_INT, pi->id,
@@ -94,7 +95,7 @@ void db_top_update_project_infos(Database* db, List* project_infos)
 void db_top_register_project_info(Database* db, ProjectInfo* project_info)
 {
     exec_query(db, "insert into project_info(id, name, sort) values (NULL, ?, ?)",
-            COLUMN_TYPE_TEXT, project_info->name,
+            COLUMN_TYPE_TEXT, string_rawstr(project_info->name),
             COLUMN_TYPE_INT, project_info->sort,
             COLUMN_TYPE_END);
     return;
