@@ -22,7 +22,7 @@ HOOK* init_hook(HOOK_MODE mode)
     HOOK* hook = xalloc(sizeof(HOOK));
     List* results;
     hook->mode = mode;
-    list_alloc(results, HOOK_RESULT);
+    list_alloc(results, HOOK_RESULT, NULL, NULL);
     hook->results = results;
     return hook;
 }
@@ -49,7 +49,7 @@ static String* create_json(String* content, Project* project, Message* message,
         String* field_a = string_new(0);
         String* name_a = string_new(0);
         String* value_a = string_new(0);
-        string_append(name_a, et->name);
+        string_append(name_a, string_rawstr(et->name));
         escape_quot(name_a);
         string_append(value_a, get_element_value(elements, et));
         escape_quot(value_a);
@@ -111,7 +111,7 @@ HOOK* exec_hook(HOOK* hook, Project* project, Message* message, List* elements,
                 sprintf(result->message, "[ERROR] hook処理(%s)でエラーが発生しました。(プラグインの読み込みに失敗しました。%s)", hook_command, dlerror());
             } else {
                 char* error;
-                int (*func)(char* base_url, char* project_id, Project* project, Message* message, List* elements, List* element_types);
+                int (*func)(char*, char*, Project*, Message*, List*, List*);
                 func = dlsym(handle, "execute");
                 if ((error = dlerror()) != NULL) {
                     d("dlsym error %s\n", error);
