@@ -6,9 +6,13 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <poll.h>
+#include "../../alloc.h"
+#include "../../conv.h"
 #include "../../data.h"
 #include "../../util.h"
+#include "../../list.h"
 #include "../../simple_string.h"
+/* #define d(...) printf(__VA_ARGS__) */
 
 static int wait(int soc, struct pollfd *target, int timeout)
 {
@@ -17,18 +21,18 @@ static int wait(int soc, struct pollfd *target, int timeout)
         ret = poll(target, 1, timeout);
         d("poll ret %d\n", ret);
         if (ret == -1 && errno != EINTR) {
-            /* 顼 */
+            /* エラー */
             return -1;
         } else if (ret == 0) {
             /* timeout */
-        d("poll timeout \n");
+            d("poll timeout \n");
             return -2;
         }
         if (target->revents & (POLLIN | POLLERR)) {
             char buf[DEFAULT_LENGTH];
             recv(soc, buf, sizeof(buf), 0);
             d("recv: %s\n", buf);
-            /* OK */
+            /* 送信準備OK */
             return 0;
         }
     }
@@ -37,15 +41,44 @@ static int send_data(int soc, char* buf)
 {
     d("send: %s\n", buf);
     send(soc, buf, strlen(buf), 0);
+    d("sended \n");
     return 0;
 }
+static char* hook_get_element_value_by_id(List* elements, const int type)
+{
+    Iterator* it;
+    d("hook_get_element_value_by_id 0 %d\n", type);
+    if (elements == NULL) return "";
+    d("hook_get_element_value_by_id 1\n");
+    d("hook_get_element_value_by_id 1 %d \n", elements->size);
+    foreach (it, elements) {
+        Element* e = it->element;
+    d("hook_get_element_value_by_id 2 \n");
+    d("hook_get_element_value_by_id 2 %s\n", string_rawstr(e->str_val));
+        if (type == e->element_type_id && string_rawstr(e->str_val) != NULL)
+            return string_rawstr(e->str_val);
+    }
+    d("hook_get_element_value_by_id 3\n");
+    return "";
+}
 static int build_header(String* buf, char* base_url, char* project_id, Project* project, Message* message, List* elements, List* element_types)
 {
+    char* subject;
+    char* subject_b64_a;
+    d("0\n");
+    subject = hook_get_element_value_by_id(elements, ELEM_ID_TITLE);
+    d("0\n");
+    d("0 %s\n", subject);
+    d("1\n");
+    subject_b64_a = xalloc(sizeof(char) * strlen(subject) * 1.5); /* base64での増加分を考慮 */
+    base64_encode((unsigned char*)subject, (unsigned char*)subject_b64_a);
+    d("3 %s\n", subject_b64_a);
     string_appendf(buf, "From: %s\r\n", FROM);
     string_appendf(buf, "To: %s\r\n", TO);
-    string_appendf(buf, "Subject: [%s:%d]Starbug1 notify.\r\n", project_id, message->id);
+    string_appendf(buf, "Subject: [%s:%d]Starbug1 notify. =?UTF-8?B?%s?=\r\n", project_id, message->id, subject_b64_a);
     string_appendf(buf, "Content-Type: text/plain;\r\n");
     string_appendf(buf, "\r\n");
+    xfree(subject_b64_a);
     return 0;
 }
 static int build_body(String* buf, char* base_url, char* project_id, Project* project, Message* message, List* elements, List* element_types)
@@ -159,113 +192,119 @@ int execute(char* base_url, char* project_id, Project* project, Message* message
     return 0;
 }
 
-/* ʲƥѴؿ */
+/* 以下、テスト用関数 */
 void conv(char*, char*);
 int main(int argc, char** argv)
 {
-    char* hostnm;
-    char* portnm;
-    struct addrinfo hints, *res, *res0;
-    int soc;
-    char hbuf[NI_MAXHOST], sbuf[NI_MAXHOST];
-    int error;
-    struct pollfd targets[2];
-    int nready, len, end;
-    char buf[1024];
-
-    if (argc < 3) {
-        fprintf(stderr, "%s host port\n", argv[0]);
-        exit(1);
-    }
-    hostnm = argv[1];
-    portnm = argv[2];
-
-    memset(&hints, 0, sizeof(hints));
-    hints.ai_socktype = SOCK_STREAM;
-    error = getaddrinfo(hostnm, portnm, &hints, &res0);
-    if (error) {
-        fprintf(stderr, "getaddrinfo: %s %s:%s\n", hostnm, portnm, gai_strerror(error));
-        return -1;
-    }
-
-    soc = -1;
-    for (res = res0; res; res = res->ai_next) {
-        error = getnameinfo(res->ai_addr, res->ai_addrlen, hbuf, sizeof(hbuf), sbuf,
-                sizeof(sbuf), NI_NUMERICHOST | NI_NUMERICSERV);
-        if (error) {
-            fprintf(stderr, "getnameinfo: %s %s:%s\n", hostnm, portnm, gai_strerror(error));
-            continue;
-        }
-        fprintf(stderr, "trying %s port %s \n", hbuf, sbuf);
-        soc = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
-        if (soc < 0) {
-            continue;
-        }
-        if (connect(soc, res->ai_addr, res->ai_addrlen) < 0) {
-            close(soc);
-            soc = -1;
-            continue;
-        } else {
-            fprintf(stderr, "connect: Success\n");
-            break;
-        }
-    }
-    freeaddrinfo(res0);
-
-    if (soc < 0) {
-        fprintf(stderr, "no destination to connect to\n");
-        return -1;
-    }
-    targets[0].fd = soc;
-    targets[0].events = POLLIN | POLLERR;
-    targets[1].fd = fileno(stdin);
-    targets[1].events = POLLIN | POLLERR;
-
-    setbuf(stdout, NULL);
-
-    end = 0;
-    do {
-        switch ((nready = poll(targets, 2, -1))) {
-            case -1:
-                if (errno != EINTR) {
-                    perror("poll");
-                    end = 1;
-                }
-                break;
-            case 0:
-                fprintf(stderr, "Timeout\n");
-                break;
-            default:
-                if (targets[0].revents & (POLLIN | POLLERR)) {
-                    if ((len = recv(soc, buf, sizeof(buf), 0)) < 0) {
-                        perror("recv");
-                        end = 1;
-                    } else if (len == 0) {
-                        fprintf(stderr, "recv: closed\n");
-                        end = 1;
-                    } else {
-                        buf[len] = '\0';
-                        fputs(buf, stdout);
-                    }
-                }
-                if (targets[1].revents & (POLLIN | POLLERR)) {
-                    if (fgets(buf, sizeof(buf), stdin) == NULL) {
-                        perror("fgets");
-                        end = 1;
-                    } else {
-                        char b[1024];
-                        strcpy(b, buf);
-                        conv(buf, b);
-                        send(soc, buf, strlen(buf), 0);
-                    }
-                }
-                break;
-        }
-    } while (end != 1);
-
-    close(soc);
-
+    char* subject = "日本語";
+    char* subject_b64_a = xalloc(sizeof(char) * strlen(subject) * 1.5); /* base64での増加分を考慮 */
+    base64_encode((unsigned char*)subject, (unsigned char*)subject_b64_a);
+    d("base64_encode: %s\n", subject_b64_a);
+    xfree(subject_b64_a);
     return 0;
+/*    char* hostnm;*/
+/*    char* portnm;*/
+/*    struct addrinfo hints, *res, *res0;*/
+/*    int soc;*/
+/*    char hbuf[NI_MAXHOST], sbuf[NI_MAXHOST];*/
+/*    int error;*/
+/*    struct pollfd targets[2];*/
+/*    int nready, len, end;*/
+/*    char buf[1024];*/
+
+/*    if (argc < 3) {*/
+/*        fprintf(stderr, "%s host port\n", argv[0]);*/
+/*        exit(1);*/
+/*    }*/
+/*    hostnm = argv[1];*/
+/*    portnm = argv[2];*/
+
+/*    memset(&hints, 0, sizeof(hints));*/
+/*    hints.ai_socktype = SOCK_STREAM;*/
+/*    error = getaddrinfo(hostnm, portnm, &hints, &res0);*/
+/*    if (error) {*/
+/*        fprintf(stderr, "getaddrinfo: %s %s:%s\n", hostnm, portnm, gai_strerror(error));*/
+/*        return -1;*/
+/*    }*/
+
+/*    soc = -1;*/
+/*    for (res = res0; res; res = res->ai_next) {*/
+/*        error = getnameinfo(res->ai_addr, res->ai_addrlen, hbuf, sizeof(hbuf), sbuf,*/
+/*                sizeof(sbuf), NI_NUMERICHOST | NI_NUMERICSERV);*/
+/*        if (error) {*/
+/*            fprintf(stderr, "getnameinfo: %s %s:%s\n", hostnm, portnm, gai_strerror(error));*/
+/*            continue;*/
+/*        }*/
+/*        fprintf(stderr, "trying %s port %s \n", hbuf, sbuf);*/
+/*        soc = socket(res->ai_family, res->ai_socktype, res->ai_protocol);*/
+/*        if (soc < 0) {*/
+/*            continue;*/
+/*        }*/
+/*        if (connect(soc, res->ai_addr, res->ai_addrlen) < 0) {*/
+/*            close(soc);*/
+/*            soc = -1;*/
+/*            continue;*/
+/*        } else {*/
+/*            fprintf(stderr, "connect: Success\n");*/
+/*            break;*/
+/*        }*/
+/*    }*/
+/*    freeaddrinfo(res0);*/
+
+/*    if (soc < 0) {*/
+/*        fprintf(stderr, "no destination to connect to\n");*/
+/*        return -1;*/
+/*    }*/
+/*    targets[0].fd = soc;*/
+/*    targets[0].events = POLLIN | POLLERR;*/
+/*    targets[1].fd = fileno(stdin);*/
+/*    targets[1].events = POLLIN | POLLERR;*/
+
+/*    setbuf(stdout, NULL);*/
+
+/*    end = 0;*/
+/*    do {*/
+/*        switch ((nready = poll(targets, 2, -1))) {*/
+/*            case -1:*/
+/*                if (errno != EINTR) {*/
+/*                    perror("poll");*/
+/*                    end = 1;*/
+/*                }*/
+/*                break;*/
+/*            case 0:*/
+/*                fprintf(stderr, "Timeout\n");*/
+/*                break;*/
+/*            default:*/
+/*                if (targets[0].revents & (POLLIN | POLLERR)) {*/
+/*                    if ((len = recv(soc, buf, sizeof(buf), 0)) < 0) {*/
+/*                        perror("recv");*/
+/*                        end = 1;*/
+/*                    } else if (len == 0) {*/
+/*                        fprintf(stderr, "recv: closed\n");*/
+/*                        end = 1;*/
+/*                    } else {*/
+/*                        buf[len] = '\0';*/
+/*                        fputs(buf, stdout);*/
+/*                    }*/
+/*                }*/
+/*                if (targets[1].revents & (POLLIN | POLLERR)) {*/
+/*                    if (fgets(buf, sizeof(buf), stdin) == NULL) {*/
+/*                        perror("fgets");*/
+/*                        end = 1;*/
+/*                    } else {*/
+/*                        char b[1024];*/
+/*                        strcpy(b, buf);*/
+/*                        conv(buf, b);*/
+/*                        send(soc, buf, strlen(buf), 0);*/
+/*                    }*/
+/*                }*/
+/*                break;*/
+/*        }*/
+/*    } while (end != 1);*/
+
+/*    close(soc);*/
+
+/*    return 0;*/
 }
 void conv(char* dest, char* src)
 {
